//=============================================================================
//Sheep Counter Main Processor Code
//2010
//=============================================================================  

#include <usart.h>
#include <rtcc.h>
#include "FSIO.h"
#include <delays.h>

//=========================================================================================================

#if defined(PIC18F4685_PIM)				// Tristan
        #pragma config OSC     = HSPLL     // Firmware must also set OSCTUNE<PLLEN> to start PLL!
		#pragma config WDT     = OFF
		#pragma config LVP    = OFF
		#pragma config FCMEN  = OFF	
#endif

#if defined(PIC18F87K22_PIM)				// Configuration bits for PIC18F87J50 FS USB Plug-In Module board
		#pragma config RETEN   = OFF   	// VREG sleep enable
        #pragma config XINST    = OFF   	// Extended instruction set
        #pragma config STVREN   = ON      	// Stack overflow reset
        //#pragma config PLLDIV   = 3         // (12 MHz crystal used on this board)
        #pragma config WDTEN    = OFF      	// Watch Dog Timer (WDT)
        #pragma config CP0      = OFF      	// Code protect
        //#pragma config CPUDIV   = OSC1      // OSC1 = divide by 1 mode
        #pragma config IESO     = OFF      	// Internal External (clock) Switchover
        #pragma config FCMEN    = OFF      	// Fail Safe Clock Monitor
        #pragma config FOSC     = HS2     // Firmware must also set OSCTUNE<PLLEN> to start PLL!
		#pragma config PLLCFG    = ON   	// X4
     //   #pragma config MSSPMSK  = MSK5
//      #pragma config PMPMX    = DEFAULT
//      #pragma config ECCPMX   = DEFAULT
        //#pragma config CCP2MX   = DEFAULT   
#endif

//=============================================================================
//Interrupt Prototypes / Code
//=============================================================================  

//void low_isr(void);
/*
* For PIC18 devices the low interrupt vector is found at
* 00000018h. The following code will branch to the
* low_interrupt_service_routine function to handle
* interrupts that occur at the low vector.
*/
//#pragma code low_vector=0x18
//void interrupt_at_low_vector(void)
//{
//_asm GOTO low_isr _endasm
//}
//#pragma code /* return to the default code section */
//#pragma interruptlow low_isr
//void low_isr (void);

//=============================================================================
//Interrupt Prototypes / Code
//=============================================================================  

void high_isr(void);
/*
* For PIC18 devices the low interrupt vector is found at
* 00000018h. The following code will branch to the
* low_interrupt_service_routine function to handle
* interrupts that occur at the low vector.
*/
#pragma code high_vector=0x08
void interrupt_at_high_vector(void)
{
_asm GOTO high_isr _endasm
}
#pragma code /* return to the default code section */
#pragma interrupt high_isr
void high_isr (void);

//=============================================================================
//Function Prototypes
//=============================================================================  

void writeHeader(void);
void init(void);
void tmr0Delay (void);
int recombine( unsigned int H, unsigned int L );
void backlightMenu(void);
void timeMenu(void);
void setMenu(void);
void setPixel(unsigned char xVal, unsigned char yVal, unsigned char onOff);
void graphEEG(char *stringPtr);
void lcdClr (void);
void drawLine(unsigned char x1Val, unsigned char y1Val, unsigned char x2Val, unsigned char y2Val, unsigned char onOff);
void setTime(void);
void setDate(void);

//=============================================================================
//Variable Declarations
//=============================================================================  

unsigned char key=0;

char send2[2] = "2";
char receiveBuffer[50]; //50 default

unsigned char debug=0;
unsigned char debounceFlag=0;
unsigned char temp=0;
unsigned char state=2;
unsigned char blah=0;
unsigned char menu=0;
signed char value=104;
signed char valueOld=104;
unsigned char inCount = 0;
unsigned char serialStart = 0;
unsigned char log = 0;
unsigned char dataLog = 0;
unsigned long int samplesDebug = 0;
char inputBuffer[64];
char *recPointer;
unsigned char dataRdy = 0;
unsigned char last = 0;
unsigned char write = 0;
unsigned char numInterrupt = 0;
signed long int interruptDebug = 0;
unsigned char oldTime=0;
unsigned char finishedLog=0;
unsigned char samples = 0;
char timeDate = 2;
char digit = 0;
char digitValue = 0;
unsigned char numFiles = 1;
char firstDigitVal = 0;
char lastDigit2 = 0;

//=============================================================================
//File System Defs
//=============================================================================  

FSFILE * pointer;
char path[30]; //30 default
char count = 30;
char * pointer2; //char default
SearchRec rec;
unsigned char attributes;
unsigned char size = 0, i; //char default
unsigned char spaceCount = 0;
char *fileName;
const rom char fileName1[10] = "File1.edf";
const rom char fileName2[10] = "File2.edf";
const rom char fileName3[10] = "File3.edf";
const rom char fileName4[10] = "File4.edf";
const rom char fileName5[10] = "File5.edf";
const rom char fileName6[10] = "File6.edf";
const rom char fileName7[10] = "File7.edf";
const rom char fileName8[10] = "File8.edf";
const rom char fileName9[10] = "File9.edf";


//=============================================================================
//Header Strings
//=============================================================================  

char CardName[7] = "EEGDAT";
char headerBIOSEMI[8]="BIOSEMI";
char header0[2]="0";
char headerELECTRODE[17]="Active Electrode";
char headerUNIT[3]="uV";
char headerDIGMAX[4]="511";
char headerDIGMIN[5]="-512";
char headerRESERVED[9]="Reserved";
char headerSPACE[2]=" ";
char headerUSER[14]="SHEEP COUNTER";
char headerTIME[9]="12.00.00";
char headerDATE[9]="01.01.10"; //<================ MOD!!   
char headerNUMBYTES[4]="512";
char headerNUMDATA[3]="-1";
char headerDURATION[2]="1";
char headerNAME[5]="Ch 1";
char headerSAMPLERATE[4]="256";
unsigned char send[17];
char timeString[7];
unsigned char sendMe[2];

//=============================================================================
//Main
//=============================================================================  

void main (void)
{

	while (!MDD_MediaDetect());

	// Initialize the library
	while (!FSInit());
						
	if (SetClockVars (2007, 7, 27, 15, 5, 26))
	{
		while(1);
	}


init();

//putrs1USART("TEST");

send[17]=0;
sendMe[1] = 0;

//=============================================================================
//Main Loop
//=============================================================================  

	timeMenu();

	while(1){
		switch(state)
		{
			case 0:
				if(debounceFlag)
				{
					if(key) state = 1;
					else if(!key) state = 2;
					INTCONbits.TMR0IE = 1;
					T0CONbits.TMR0ON = 0;
					debounceFlag = 0;	
				}
			break;
			case 1:
				if(!PORTAbits.RA4 & !PORTAbits.RA5 & !PORTJbits.RJ4 & !PORTJbits.RJ5)
				{
					key = 0;
					TMR0H = 250;
					TMR0L = 0;
					state = 0;
					INTCONbits.TMR0IE = 1;
					T0CONbits.TMR0ON = 1; //turn on timer3
				}
			break;
			case 2:
			///////////////////////////////////////////////////////////////////////////
			//Down
			///////////////////////////////////////////////////////////////////////////
			if(PORTAbits.RA4 & !key) 
			{
				if(menu == 3) value -= 8;
				else if(menu == 1) log ^= 1;
				else if(menu == 2) timeDate--;
				else if(menu == 4) 
				{
					if(digit == 0)
					{
						RTCCFGbits.RTCPTR0 = 1; // RTCVALH = minutes RTCVALL = seconds 
						RTCCFGbits.RTCPTR1 = 0;
		
						digitValue = (((RTCVALL>>4) & 0b00001111)-1); //Hours high
						if(digitValue < 0) digitValue = 0;

						firstDigitVal = digitValue;

						_asm movlw 0x55 _endasm
						_asm movwf EECON2,0 _endasm
						_asm movlw 0xAA _endasm
						_asm movwf EECON2,0 _endasm 
						
						RTCCFGbits.RTCWREN = 1; 

						RTCCFGbits.RTCPTR0 = 1; // RTCVALH = minutes RTCVALL = seconds 
						RTCCFGbits.RTCPTR1 = 0;

						RTCVALL &= 0b00001111;

						RTCCFGbits.RTCPTR0 = 1; // RTCVALH = minutes RTCVALL = seconds 
						RTCCFGbits.RTCPTR1 = 0;

						RTCVALL |= ((digitValue<<4) & 0b11110000); //Hours high

						_asm movlw 0x55 _endasm
						_asm movwf EECON2,0 _endasm
						_asm movlw 0xAA _endasm
						_asm movwf EECON2,0 _endasm 
						
						RTCCFGbits.RTCWREN = 0; 

					}
					else if(digit == 1)
					{
						RTCCFGbits.RTCPTR0 = 1; // RTCVALH = minutes RTCVALL = seconds 
						RTCCFGbits.RTCPTR1 = 0;
		
						digitValue = (((RTCVALL & 0b00001111))-1); //Hours high
						if(digitValue < 0) digitValue = 0;



						_asm movlw 0x55 _endasm
						_asm movwf EECON2,0 _endasm
						_asm movlw 0xAA _endasm
						_asm movwf EECON2,0 _endasm 
						
						RTCCFGbits.RTCWREN = 1; 

						RTCCFGbits.RTCPTR0 = 1; // RTCVALH = minutes RTCVALL = seconds 
						RTCCFGbits.RTCPTR1 = 0;

						RTCVALL &= 0b11110000;

						RTCCFGbits.RTCPTR0 = 1; // RTCVALH = minutes RTCVALL = seconds 
						RTCCFGbits.RTCPTR1 = 0;

						RTCVALL |= ((digitValue) & 0b00001111); //Hours high

						_asm movlw 0x55 _endasm
						_asm movwf EECON2,0 _endasm
						_asm movlw 0xAA _endasm
						_asm movwf EECON2,0 _endasm 
						
						RTCCFGbits.RTCWREN = 0; 

					}
					else if(digit == 2)
					{
						RTCCFGbits.RTCPTR0 = 0; // RTCVALH = minutes RTCVALL = seconds 
						RTCCFGbits.RTCPTR1 = 0;
		
						digitValue = (((RTCVALH>>4) & 0b00001111)-1); //Hours high
						if(digitValue < 0) digitValue = 0;

						_asm movlw 0x55 _endasm
						_asm movwf EECON2,0 _endasm
						_asm movlw 0xAA _endasm
						_asm movwf EECON2,0 _endasm 
						
						RTCCFGbits.RTCWREN = 1; 

						RTCVALH &= 0b00001111;

						RTCVALH |= ((digitValue<<4) & 0b11110000); //Hours high

						_asm movlw 0x55 _endasm
						_asm movwf EECON2,0 _endasm
						_asm movlw 0xAA _endasm
						_asm movwf EECON2,0 _endasm 
						
						RTCCFGbits.RTCWREN = 0; 
					}
					else if(digit == 3)
					{
						RTCCFGbits.RTCPTR0 = 0; // RTCVALH = minutes RTCVALL = seconds 
						RTCCFGbits.RTCPTR1 = 0;
		
						digitValue = (((RTCVALH & 0b00001111))-1); //Hours high
						if(digitValue < 0) digitValue = 0;

						_asm movlw 0x55 _endasm
						_asm movwf EECON2,0 _endasm
						_asm movlw 0xAA _endasm
						_asm movwf EECON2,0 _endasm 
						
						RTCCFGbits.RTCWREN = 1; 

						RTCVALH &= 0b11110000;

						RTCVALH |= ((digitValue) & 0b00001111); //Hours high

						_asm movlw 0x55 _endasm
						_asm movwf EECON2,0 _endasm
						_asm movlw 0xAA _endasm
						_asm movwf EECON2,0 _endasm 
						
						RTCCFGbits.RTCWREN = 0; 

					}
				} 
				tmr0Delay();
			}
			///////////////////////////////////////////////////////////////////////////
			//Last Menu
			///////////////////////////////////////////////////////////////////////////
			else if(PORTAbits.RA5 & !key & !log)
			{
				if(menu == 2)
				{
					if(timeDate == 2) menu--;
					else if(timeDate == 0) menu = 5;
					else if(timeDate == 1) menu = 4;
				}
				else if((menu == 4) | (menu == 5))
				{
					digit--;
					if(digit < 0)
					{
						digit = 0;
					}
				}
				else 
				{
					menu--;
					if(menu == 255) menu = 3;
				}
				tmr0Delay();
			}
			///////////////////////////////////////////////////////////////////////////
			//Next Menu
			///////////////////////////////////////////////////////////////////////////
			else if(PORTJbits.RJ4 & !key & !log)
			{
				if(menu == 2)
				{
					if(timeDate == 2) menu++;
					else if(timeDate == 0) menu = 5;
					else if(timeDate == 1) menu = 4;
				}
				else if((menu == 4) | (menu == 5))
				{
					digit++;
					if(digit >= 4)
					{
						digit = 0;
						menu = 2;
					}
				}
				else
				{ 
					menu++;
					if(menu == 4) menu=0;
				}	
				tmr0Delay();
			}
			///////////////////////////////////////////////////////////////////////////
			//UP
			///////////////////////////////////////////////////////////////////////////
			else if(PORTJbits.RJ5 & !key & !log)
			{
				if(menu == 3) value += 8;
				else if(menu == 2) timeDate++;
				else if(menu == 4) 
				{
					if(digit == 0)
					{
						RTCCFGbits.RTCPTR0 = 1; // RTCVALH = minutes RTCVALL = seconds 
						RTCCFGbits.RTCPTR1 = 0;
		
						digitValue = (((RTCVALL>>4) & 0b00001111)+1); //Hours high
						if(digitValue > 2) digitValue = 2;
						if((digitValue > 1) & ((lastDigit2 - 0x30) > 3))
						{
							//////////////////////////////////////////////////////////////!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
							RTCCFGbits.RTCPTR0 = 1; // RTCVALH = minutes RTCVALL = seconds 
							RTCCFGbits.RTCPTR1 = 0;
				
							_asm movlw 0x55 _endasm
							_asm movwf EECON2,0 _endasm
							_asm movlw 0xAA _endasm
							_asm movwf EECON2,0 _endasm 
							
							RTCCFGbits.RTCWREN = 1; 
	
							RTCCFGbits.RTCPTR0 = 1; // RTCVALH = minutes RTCVALL = seconds 
							RTCCFGbits.RTCPTR1 = 0;
	
							RTCVALL &= 0b11110000;
	
							RTCCFGbits.RTCPTR0 = 1; // RTCVALH = minutes RTCVALL = seconds 
							RTCCFGbits.RTCPTR1 = 0;
	
							RTCVALL |= ((3) & 0b00001111); //Hours high
	
							_asm movlw 0x55 _endasm
							_asm movwf EECON2,0 _endasm
							_asm movlw 0xAA _endasm
							_asm movwf EECON2,0 _endasm 
							
							RTCCFGbits.RTCWREN = 0; 
						}
						
						/////////////////////////////////////////////////////////////!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
						firstDigitVal = digitValue;

						_asm movlw 0x55 _endasm
						_asm movwf EECON2,0 _endasm
						_asm movlw 0xAA _endasm
						_asm movwf EECON2,0 _endasm 
						
						RTCCFGbits.RTCWREN = 1; 

						RTCCFGbits.RTCPTR0 = 1; // RTCVALH = minutes RTCVALL = seconds 
						RTCCFGbits.RTCPTR1 = 0;

						RTCVALL &= 0b00001111;

						RTCCFGbits.RTCPTR0 = 1; // RTCVALH = minutes RTCVALL = seconds 
						RTCCFGbits.RTCPTR1 = 0;

						RTCVALL |= ((digitValue<<4) & 0b11110000); //Hours high

						_asm movlw 0x55 _endasm
						_asm movwf EECON2,0 _endasm
						_asm movlw 0xAA _endasm
						_asm movwf EECON2,0 _endasm 
						
						RTCCFGbits.RTCWREN = 0; 

					}
					else if(digit == 1)
					{
						RTCCFGbits.RTCPTR0 = 1; // RTCVALH = minutes RTCVALL = seconds 
						RTCCFGbits.RTCPTR1 = 0;
						digitValue = (((RTCVALL & 0b00001111))+1); //Hours high
						if(firstDigitVal < 2)
						{
							if(digitValue > 9) digitValue = 9;
						}
						if(firstDigitVal > 1)
						{
							if(digitValue > 3) digitValue = 3;
						}

						_asm movlw 0x55 _endasm
						_asm movwf EECON2,0 _endasm
						_asm movlw 0xAA _endasm
						_asm movwf EECON2,0 _endasm 
						
						RTCCFGbits.RTCWREN = 1; 

						RTCCFGbits.RTCPTR0 = 1; // RTCVALH = minutes RTCVALL = seconds 
						RTCCFGbits.RTCPTR1 = 0;

						RTCVALL &= 0b11110000;

						RTCCFGbits.RTCPTR0 = 1; // RTCVALH = minutes RTCVALL = seconds 
						RTCCFGbits.RTCPTR1 = 0;

						RTCVALL |= ((digitValue) & 0b00001111); //Hours high

						_asm movlw 0x55 _endasm
						_asm movwf EECON2,0 _endasm
						_asm movlw 0xAA _endasm
						_asm movwf EECON2,0 _endasm 
						
						RTCCFGbits.RTCWREN = 0; 

					}
					else if(digit == 2)
					{
						RTCCFGbits.RTCPTR0 = 0; // RTCVALH = minutes RTCVALL = seconds 
						RTCCFGbits.RTCPTR1 = 0;
		
						digitValue = (((RTCVALH>>4) & 0b00001111)+1); //Hours high
						if(digitValue > 5) digitValue = 5;

						_asm movlw 0x55 _endasm
						_asm movwf EECON2,0 _endasm
						_asm movlw 0xAA _endasm
						_asm movwf EECON2,0 _endasm 
						
						RTCCFGbits.RTCWREN = 1; 

						RTCVALH &= 0b00001111;

						RTCVALH |= ((digitValue<<4) & 0b11110000); //Hours high

						_asm movlw 0x55 _endasm
						_asm movwf EECON2,0 _endasm
						_asm movlw 0xAA _endasm
						_asm movwf EECON2,0 _endasm 
						
						RTCCFGbits.RTCWREN = 0; 
					}
					else if(digit == 3)
					{
						RTCCFGbits.RTCPTR0 = 0; // RTCVALH = minutes RTCVALL = seconds 
						RTCCFGbits.RTCPTR1 = 0;
		
						digitValue = (((RTCVALH & 0b00001111))+1); //Hours high
						if(digitValue > 9) digitValue = 9;

						_asm movlw 0x55 _endasm
						_asm movwf EECON2,0 _endasm
						_asm movlw 0xAA _endasm
						_asm movwf EECON2,0 _endasm 
						
						RTCCFGbits.RTCWREN = 1; 

						RTCVALH &= 0b11110000;

						RTCVALH |= ((digitValue) & 0b00001111); //Hours high

						_asm movlw 0x55 _endasm
						_asm movwf EECON2,0 _endasm
						_asm movlw 0xAA _endasm
						_asm movwf EECON2,0 _endasm 
						
						RTCCFGbits.RTCWREN = 0; 

					}
				} 
				tmr0Delay();
			}
			
//=============================================================================
//Menu Dispay
//=============================================================================  

			if((key == 1)|((menu == 0) & (RTCVALH != oldTime))) //Menu Display
			{
				switch(menu)
				{
					case 0: //TIME DISPLAY
					timeMenu();
					break;
					case 1: //EEG GRAPH
						if(!log)
						{
							PIE1bits.RC1IE = 0;
							putc2USART(0x7C); //CMD
							while(Busy2USART());
							putc2USART(0x00);  //CLR
							while(Busy2USART());
							putrs2USART("EEG Graph");
						}
						if(log)
						{
							putc2USART(0x7C); //CMD
							while(Busy2USART());
							putc2USART(0x00);  //CLR
							while(Busy2USART());
							writeHeader();

							Open1USART( USART_TX_INT_OFF  &
	
						         USART_RX_INT_OFF &
						
						         USART_ASYNCH_MODE &
						
						         USART_EIGHT_BIT   &
						
						         USART_CONT_RX     &
						
						         USART_BRGH_HIGH, 
	
							     32    ); //68 //57.6 kbaud @ 64Mhz

							dataLog = 1;
							PIR1bits.RCIF=0;
							PIE1bits.RC1IE = 1; //enable serial receive interrupt
						}
						else if(!log & dataLog & !finishedLog) //CLOSE FILE
						{
							PIE1bits.RC1IE = 0;
							Close1USART();
							dataRdy = 0;
							numInterrupt = 0;
							last = 0;
							dataLog = 0;
								// FSftell returns the file's current position
							//FSftell (pointer);
							
						
							
							// FSfseek sets the position one byte before the end
							// It can also set the position of a file forward from the
							// beginning or forward from the current position
							if (FSfseek(pointer, 1, SEEK_END))
							while(1);
							
							// Close the file
							if (FSfclose (pointer))
							while(1);

						}
							
					break;
					case 2: //SET TIME/ALARM
						setMenu();
					break;
					case 3: //BACKLIGHT LEVEL
					backlightMenu();
					break;
					case 4:
					setTime();
					break;
					case 5:
					setDate();
 					break;	
					}
			}
			break;
		}
		if(dataRdy & (samples >= 8))
		{
			sendMe[0] = inputBuffer[(write+2)];
			send[0] = sendMe[0];
		
				if (FSfwrite ((void *)sendMe,1, 1, pointer) != 1)
				{
					while(1);	
				}
		
			sendMe[0] = inputBuffer[(write+3)]; 
			send[1]=sendMe[0];
		
				if (FSfwrite ((void *)sendMe,1, 1, pointer) != 1)
				{
					while(1);	
				}
		
			sendMe[0] = inputBuffer[(write+6)];
		
				if (FSfwrite ((void *)sendMe,1, 1, pointer) != 1)
				{
					while(1);	
				}
		
			sendMe[0] = inputBuffer[(write+7)];
		
				if (FSfwrite ((void *)sendMe,1, 1, pointer) != 1)
				{
					while(1);	
				}
		
			sendMe[0] = inputBuffer[(write+10)];
		
				if (FSfwrite ((void *)sendMe,1, 1, pointer) != 1)
				{
					while(1);	
				}
		
			sendMe[0] = inputBuffer[(write+11)];
		
				if (FSfwrite ((void *)sendMe,1, 1, pointer) != 1)
				{
					while(1);	
				}
		
			sendMe[0] = inputBuffer[(write+14)];
			send[14]=sendMe[0];
		
				if (FSfwrite ((void *)sendMe,1, 1, pointer) != 1)
				{
					while(1);	
				}
		
			sendMe[0] = inputBuffer[(write+15)];
			send[15] = sendMe[0];
		
				if (FSfwrite ((void *)sendMe,1, 1, pointer) != 1)
				{
					while(1);	
				}
			
			sendMe[0] = inputBuffer[(write+18)];
			
				if (FSfwrite ((void *)sendMe,1, 1, pointer) != 1)
				{
					while(1);	
				}
		
			sendMe[0] = inputBuffer[(write+19)];
		
				if (FSfwrite ((void *)sendMe,1, 1, pointer) != 1)
				{
					while(1);	
				}
			
			sendMe[0] = inputBuffer[(write+22)];
				
				if (FSfwrite ((void *)sendMe,1, 1, pointer) != 1)
				{
					while(1);	
				}
		
			sendMe[0] = inputBuffer[(write+23)];
		
				if (FSfwrite ((void *)sendMe,1, 1, pointer) != 1)
				{
					while(1);	
				}
			
			sendMe[0] = inputBuffer[(write+26)];
				
				if (FSfwrite ((void *)sendMe,1, 1, pointer) != 1)
				{
					while(1);	
				}
			
			sendMe[0] = inputBuffer[(write+27)];
				
				if (FSfwrite ((void *)sendMe,1, 1, pointer) != 1)
				{
					while(1);	
				}
			
			sendMe[0] = inputBuffer[(write+30)];
			
				if (FSfwrite ((void *)sendMe,1, 1, pointer) != 1)
				{
					while(1);	
				}
		
			sendMe[0] = inputBuffer[(write+31)];
				
				if (FSfwrite ((void *)sendMe,1, 1, pointer) != 1)
				{
					while(1);	
				}
		

			graphEEG(&send[0]); //Graph
			samplesDebug += 8;
			samples -= 8;
			write += 32;
			if(write == 64)
			{
				write = 0;
			}
			//dataRdy = 0;
		}
	}
}

void graphEEG(char *stringPtr)
{
	static unsigned char graphCount;
	static unsigned char lastX;
	static unsigned char lastY;
	unsigned int valDivH, valDivL;
	unsigned int endVal;
	unsigned char endValChar1, endValChar2;
	unsigned char graphCount2;


		valDivH = (stringPtr[1] & 0b11111111);
		valDivL = (stringPtr[0] & 0b11111111);

		endVal = ((valDivH<<8) | valDivL); //combine two bytes into int value

		endVal += 32768;
		
		endVal /= 1024; //convert value to 64 pixels 1024 for 16 bit, 16 for 10 bit
		endValChar1 = 63 - (unsigned char)endVal;

		valDivH = (stringPtr[15] & 0b11111111);
		valDivL = (stringPtr[14] & 0b11111111);
		endVal = (((valDivH<<8) | valDivL)); //combine two bytes into int value
		
		endVal += 32768;

		endVal /= 1024; //convert value to 64 pixels
		endValChar2 = 63 - (unsigned char)endVal;


		if(graphCount >= 127)
		{ 
			lcdClr();
			graphCount = 0;
			drawLine(0, lastY, graphCount, endValChar1, 1);

			drawLine(graphCount, endValChar1, (graphCount + 1) , endValChar2, 1);
		}
		else
		{
			drawLine(lastX, lastY, graphCount, endValChar1, 1);

			drawLine(graphCount, endValChar1, (graphCount + 1) , endValChar2, 1);
		}
		lastX = (graphCount + 1);
		lastY = endValChar1;		

		graphCount += 2;
	
}

void lcdClr (void)
{
	putc2USART(0x7C); //CMD
	while(Busy2USART());
	putc2USART(0x00);  //CLR
	while(Busy2USART());
}

void drawLine(unsigned char x1Val, unsigned char y1Val, unsigned char x2Val, unsigned char y2Val, unsigned char onOff)
{
		putc2USART(0x7C); //CMD
		while(Busy2USART());
		putc2USART(0x0C);  //set pixel cmd
		while(Busy2USART());
		putc2USART(x1Val);  //set xVal
		while(Busy2USART());
		putc2USART(y1Val);  //set yVal
		while(Busy2USART());
		putc2USART(x2Val);  //set xVal
		while(Busy2USART());
		putc2USART(y2Val);  //set yVal
		while(Busy2USART());
		putc2USART(onOff);  //set pixel on/off
		while(Busy2USART());	
}

void setPixel(unsigned char xVal, unsigned char yVal, unsigned char onOff)
{
		putc2USART(0x7C); //CMD
		while(Busy2USART());
		putc2USART(0x10);  //set pixel cmd
		while(Busy2USART());
		putc2USART(xVal);  //set xVal
		while(Busy2USART());
		putc2USART(yVal);  //set yVal
		while(Busy2USART());
		putc2USART(onOff);  //set pixel on/off
		while(Busy2USART());	
}

//<========================================================================================	
//<========================================================================================	
//<========================================================================================	
//<========================================================================================	

void setTime(void)
{
	putc2USART(0x7C); //CMD
	while(Busy2USART());
	putc2USART(0x00);  //CLR
	while(Busy2USART());
	putrs2USART("  Set Time");
	while(Busy2USART());
	putc2USART(0xD); //Carrige return
	while(Busy2USART());
	putc2USART(0xD); //Carrige return
	while(Busy2USART());

	RTCCFGbits.RTCPTR0 = 1; // RTCVALH = minutes RTCVALL = seconds 
	RTCCFGbits.RTCPTR1 = 0;

	timeString[1] = ((RTCVALL & 0b00001111) + 0x30); //Hours low
	timeString[0] = (((RTCVALL>>4) & 0b00001111) + 0x30); //Hours high
	lastDigit2 = timeString[1];

	RTCCFGbits.RTCPTR0 = 0; 
	RTCCFGbits.RTCPTR1 = 0;

	timeString[4] = ((RTCVALH & 0b00001111) + 0x30); //Minutes low
	timeString[3] = (((RTCVALH>>4) & 0b00001111) + 0x30); //Minutes high

	putrs2USART("  ");	
	while(Busy2USART());
	puts2USART(timeString);
	while(Busy2USART());
	if(digit == 0) drawLine(12, 25, 16, 25, 1);
	else if(digit == 1)
	{ 
		drawLine(18, 25, 22, 25, 1);
	
	} 
	else if(digit == 2) drawLine(30, 25, 34, 25, 1);
	else if(digit == 3) drawLine(36, 25, 40, 25, 1);
	while(Busy2USART());
}

void setDate(void)
{
	putc2USART(0x7C); //CMD
	while(Busy2USART());
	putc2USART(0x00);  //CLR
	while(Busy2USART());
	putrs2USART("  Set Date");
	while(Busy2USART());
}

void setMenu(void)
{
	putc2USART(0x7C); //CMD
	while(Busy2USART());
	putc2USART(0x00);  //CLR
	while(Busy2USART());
	putrs2USART("  Time / Date menu");
	while(Busy2USART());
	putc2USART(0xD); //Carrige return
	while(Busy2USART());
	putc2USART(0xD); //Carrige return
	while(Busy2USART());
	putrs2USART(" Change Menu");
	while(Busy2USART());
	putc2USART(0xD); //Carrige return
	while(Busy2USART());
	putc2USART(0xD); //Carrige return
	while(Busy2USART());
	putrs2USART(" Set Time");
	while(Busy2USART());
	putc2USART(0xD); //Carrige return
	while(Busy2USART());
	putc2USART(0xD); //Carrige return
	while(Busy2USART());
	putrs2USART(" Set Date");
	while(Busy2USART());
	Delay1KTCYx(10);
	if(timeDate > 2) timeDate = 2;
	else if(timeDate < 0) timeDate = 0;
	if(timeDate == 2)
	{
		putc2USART(0x7C); //CMD
		while(Busy2USART());
		putc2USART(0x0F); //BOX
		while(Busy2USART());
		putc2USART(0); //X1
		while(Busy2USART());
		putc2USART(13); //Y1
		while(Busy2USART());
		putc2USART(127); //X2
		while(Busy2USART());
		putc2USART(25); //Y2
		while(Busy2USART());
		putc2USART(0x01);
	}
	else if(timeDate == 1)
	{
		putc2USART(0x7C); //CMD
		while(Busy2USART());
		putc2USART(0x0F); //BOX
		while(Busy2USART());
		putc2USART(0); //X1
		while(Busy2USART());
		putc2USART(29); //Y1
		while(Busy2USART());
		putc2USART(127); //X2
		while(Busy2USART());
		putc2USART(41); //Y2
		while(Busy2USART());
		Delay1KTCYx(1);
		putc2USART(0x01);
	}
	else if(timeDate == 0)
	{
		putc2USART(0x7C); //CMD
		while(Busy2USART());
		putc2USART(0x0F); //BOX
		while(Busy2USART());
		putc2USART(0); //X1
		while(Busy2USART());
		putc2USART(45); //Y1
		while(Busy2USART());
		putc2USART(127); //X2
		while(Busy2USART());
		putc2USART(57); //Y2
		while(Busy2USART());
		putc2USART(0x01);
	}
 
}

void timeMenu(void)
{
	putc2USART(0x7C); //CMD
	while(Busy2USART());
	putc2USART(0x00);  //CLR
	Delay1KTCYx(2);
	putc2USART(0x7C); //CMD
	while(Busy2USART());
	putc2USART(0x08);  //CLR
	while(Busy2USART());

	RTCCFGbits.RTCPTR0 = 1; // RTCVALH = minutes RTCVALL = seconds 
	RTCCFGbits.RTCPTR1 = 0;

	timeString[1] = ((RTCVALL & 0b00001111) + 0x30); //Hours low
	timeString[0] = (((RTCVALL>>4) & 0b00001111) + 0x30); //Hours high

	RTCCFGbits.RTCPTR0 = 0; 
	RTCCFGbits.RTCPTR1 = 0;

	timeString[4] = ((RTCVALH & 0b00001111) + 0x30); //Minutes low
	timeString[3] = (((RTCVALH>>4) & 0b00001111) + 0x30); //Minutes high

	oldTime = RTCVALH;

	putrs2USART("   ");	
	while(Busy2USART());
	puts2USART(timeString);

	Delay1KTCYx(2);
	putc2USART(0x7C); //CMD
	while(Busy2USART());
	putc2USART(0x08);  //Big font toggle = 0;
	while(Busy2USART());
	//Delay1KTCYx(200);
	//putc2USART(0xD); //Carrige return
	//while(Busy2USART());
	//putrs2USART("Friday, November 19");
	
}

void backlightMenu(void)
{
	if(value == valueOld)
	{
		putc2USART(0x7C); //CMD
		while(Busy2USART());
		putc2USART(0x00);  //CLR
		while(Busy2USART());
		putrs2USART("   Backlight Level");
		while(Busy2USART());
		putc2USART(0xD);
		while(Busy2USART());
		putc2USART(0xD);
		while(Busy2USART());
		putrs2USART("Increase: UP");         
		while(Busy2USART());
		putc2USART(0xD); //Carrige return
		while(Busy2USART());
		putrs2USART("Decrease: DOWN");
		while(Busy2USART());
	}
	else if(value != valueOld) //Update Backlight level
	{
		if(value > 104) value = 104;
		else if( value < 0 ) value = 0;
		Delay10KTCYx(2);
		putc2USART(0x7C); //CMD
		while(Busy2USART());
		Delay10KTCYx(2);
		putc2USART(0x02); 
		while(Busy2USART());
		putc2USART(value);  //CLR
		Delay10KTCYx(2);
		valueOld = value;
		Delay10KTCYx(20);
	}
}

void tmr0Delay (void)
{
	debounceFlag = 0;
	key = 1;
	TMR0H = 250; 
	TMR0L = 0;
	state = 0;
	debug++;
	INTCONbits.TMR0IE = 1;
	T0CONbits.TMR0ON = 1; //turn on timer0	
}

/*
void low_isr(void)
{

}
*/

void high_isr (void)
{
	if ((INTCONbits.T0IE && INTCONbits.T0IF)) //are TMR0 interrupts enabled and
	//is the TMR0 interrupt flag set?
	{
		//to allow subsequent interrupts
		debounceFlag = 1;
		T0CONbits.TMR0ON = 0;
		INTCONbits.TMR0IE = 0;
		INTCONbits.T0IF=0; //TMR0 interrupt flag must be cleared in software
	}
	else if(PIR1bits.RC1IF & !key)//INCOMING DATA
	{	
		if(RCREG1 == 128)
		{
			while(BAUDCON1bits.RCIDL);
			while(!BAUDCON1bits.RCIDL);
			inputBuffer[last] = Read1USART();
			while(BAUDCON1bits.RCIDL);
			while(!BAUDCON1bits.RCIDL);
			inputBuffer[last + 1] = Read1USART();
			while(BAUDCON1bits.RCIDL);
			while(!BAUDCON1bits.RCIDL);
			inputBuffer[last + 2] = Read1USART();
			while(BAUDCON1bits.RCIDL);
			while(!BAUDCON1bits.RCIDL);
			inputBuffer[last + 3] = Read1USART();
	
			if((inputBuffer[last] == 64) & (inputBuffer[(last + 1)] == 64))
			{
				last += 4;
				if(last == 64) last = 0;
	
				numInterrupt++;
				interruptDebug++;
				samples++;
				if(numInterrupt == 8)
				{
					dataRdy = 1;
					numInterrupt = 0;
				}
			}
			
		}
	PIR1bits.RC1IF = 0;	
	}
		
}


void init (void)
{
	TRISAbits.TRISA5 = 1;
	TRISAbits.TRISA4 = 1;
	TRISJbits.TRISJ4 = 1;
	TRISJbits.TRISJ5 = 1;
	
	
	ANCON0 = 0;
	ANCON1 = 0;
	
	T0CONbits.T08BIT = 0; //16 bit
	T0CONbits.T0CS = 0; //use instruction clock
	T0CONbits.PSA = 0; //use prescaler
	T0CONbits.T0PS = 0b111; //prescaler = 256
	
	T0CONbits.T08BIT = 0; //16 bit
	T0CONbits.T08BIT = 0; //16 bit
	
	TMR0H = 0xFF; //start
	TMR0L = 0xFF; //start
	T0CONbits.TMR0ON = 0;
	
	INTCONbits.PEIE = 1; //enable periphrial interrupts
	PIR1bits.RC1IF = 0;
	PIE1bits.RC1IE = 0;
	INTCONbits.T0IF=0; //clear interrupt
	INTCONbits.T0IE=0; //enable timer0 interrupt
	INTCONbits.GIE=1; //global interrupt enable

//RTC///////////////

_asm movlw 0x55 _endasm
_asm movwf EECON2,0 _endasm
_asm movlw 0xAA _endasm
_asm movwf EECON2,0 _endasm 

RTCCFGbits.RTCWREN = 1;  
RTCCFGbits.RTCEN = 1; //enable RTC

RTCCFGbits.RTCPTR0 = 0; // RTCVALH = minutes RTCVALL = seconds 
RTCCFGbits.RTCPTR1 = 1;

RTCVALH =  0x11; //Set Month
RTCVALL = 0x25; //Set Day

RTCCFGbits.RTCPTR0 = 1; // RTCVALH = minutes RTCVALL = seconds 
RTCCFGbits.RTCPTR1 = 1;

RTCVALL = 0x10;//Set year

RTCCFGbits.RTCPTR0 = 1; // RTCVALH = minutes RTCVALL = seconds 
RTCCFGbits.RTCPTR1 = 0;

//RTCVALH =  0x24; //Set Weekday
RTCVALL = 0x12; //Set Hours


RTCCFGbits.RTCPTR0 = 0; // RTCVALH = minutes RTCVALL = seconds 
RTCCFGbits.RTCPTR1 = 0;

RTCVALH = 0x00;//set minutes
RTCVALL = 0; //set seconds

timeString[2] = ':';
timeString[5] = ' ';
timeString[6] = 0;

//=============================================================================
	
	RCON = 0;

	Open2USART( USART_TX_INT_OFF  &
	
	            USART_RX_INT_OFF  &
	
	            USART_ASYNCH_MODE &
	
	            USART_EIGHT_BIT   &
	
	            USART_CONT_RX     &
	
	            USART_BRGH_HIGH,
	
	            32           ); //115.2 kbaud @ 64Mhz

	T1CONbits.TMR1ON = 0;
	T2CONbits.TMR2ON = 0;
	T3CONbits.TMR3ON = 0;
}
void writeHeader ( void )
{
	//=============================================================================
	//Create File
	//=============================================================================  

	if(numFiles == 1)
	{
		pointer = FSfopenpgm (&fileName1, "w");
	}
	else if(numFiles == 2)
	{
		pointer = FSfopenpgm (&fileName2, "w");
	}
	else if(numFiles == 3)
	{
		pointer = FSfopenpgm (&fileName3, "w");
	}
	else if(numFiles == 4)
	{
		pointer = FSfopenpgm (&fileName4, "w");
	}
	else if(numFiles == 5)
	{
		pointer = FSfopenpgm (&fileName5, "w");
	}
	else if(numFiles == 6)
	{
		pointer = FSfopenpgm (&fileName6, "w");
	}
	else if(numFiles == 7)
	{
		pointer = FSfopenpgm (&fileName7, "w");
	}
	else if(numFiles == 8)
	{
		pointer = FSfopenpgm (&fileName8, "w");
	}
	else if(numFiles == 9)
	{
		pointer = FSfopenpgm (&fileName9, "w");
	}

	numFiles++;
	if(numFiles > 9) numFiles = 1;
	
	if (pointer == NULL)
	{
		while(1);
	}
	
	//=============================================================================
	//Write "0"
	//=============================================================================  
	if (FSfwrite ((void *)header0, 1, 1, pointer) != 1)
	{
		while(1);	
	}

		// FSftell returns the file's current position
	debug = FSftell (pointer);


	//=============================================================================
	//Write Spaces
	//============================================================================= 
	for(spaceCount = 0; spaceCount <= 6; spaceCount++)
	{ 
		if (FSfwrite ((void *)headerSPACE,1, 1, pointer) != 1)
		{
			while(1);	
		}
	}

	//=============================================================================
	//Write USER
	//=============================================================================  
	if (FSfwrite ((void *)headerUSER,1, 13, pointer) != 13)
	{
		while(1);	
	}

	//=============================================================================
	//Write Spaces
	//============================================================================= 
	for(spaceCount = 0; spaceCount <= 66; spaceCount++)
	{ 
		if (FSfwrite ((void *)headerSPACE,1, 1, pointer) != 1)
		{
			while(1);	
		}
	}

	//=============================================================================
	//Write USER
	//=============================================================================  
	if (FSfwrite ((void *)headerUSER,1, 13, pointer) != 13)
	{
		while(1);	
	}

	//=============================================================================
	//Write Spaces
	//============================================================================= 
	for(spaceCount = 0; spaceCount <= 66; spaceCount++)
	{ 
		if (FSfwrite ((void *)headerSPACE,1, 1, pointer) != 1)
		{
			while(1);	
		}
	}

	//=============================================================================
	//Write Start Date
	//============================================================================= 
	if (FSfwrite ((void *)headerDATE,1, 8, pointer) != 8)
	{
		while(1);	
	}

	//=============================================================================
	//Write Start Time
	//============================================================================= 
	if (FSfwrite ((void *)headerTIME,1, 8, pointer) != 8)
	{
		while(1);	
	}

	//=============================================================================
	//Number of bytes in header
	//============================================================================= 
	if (FSfwrite ((void *)headerNUMBYTES,1, 3, pointer) != 3)
	{
		while(1);	
	}

	//=============================================================================
	//Write Spaces
	//============================================================================= 
	for(spaceCount = 0; spaceCount <= 4; spaceCount++)
	{ 
		if (FSfwrite ((void *)headerSPACE,1, 1, pointer) != 1)
		{
			while(1);	
		}
	}

	//=============================================================================
	//Write BIOSEMI
	//=============================================================================  
	if (FSfwrite ((void *)headerBIOSEMI,1, 7, pointer) != 7)
	{
		while(1);	
	}

	//=============================================================================
	//Write Spaces
	//============================================================================= 
	for(spaceCount = 0; spaceCount <= 36; spaceCount++)
	{ 
		if (FSfwrite ((void *)headerSPACE,1, 1, pointer) != 1)
		{
			while(1);	
		}
	}

	//=============================================================================
	//Write NUMDATA
	//=============================================================================  
	if (FSfwrite ((void *)headerNUMDATA,1, 2, pointer) != 2)
	{
		while(1);	
	}

	//=============================================================================
	//Write Spaces
	//============================================================================= 
	for(spaceCount = 0; spaceCount <= 5; spaceCount++)
	{ 
		if (FSfwrite ((void *)headerSPACE,1, 1, pointer) != 1)
		{
			while(1);	
		}
	}

	//=============================================================================
	//Write duration
	//=============================================================================  
	if (FSfwrite ((void *)headerDURATION,1, 1, pointer) != 1)
	{
		while(1);	
	}

	//=============================================================================
	//Write Spaces
	//============================================================================= 
	for(spaceCount = 0; spaceCount <= 6; spaceCount++)
	{ 
		if (FSfwrite ((void *)headerSPACE,1, 1, pointer) != 1)
		{
			while(1);	
		}
	}

	//=============================================================================
	//Write Number of Channels
	//=============================================================================  
	if (FSfwrite ((void *)headerDURATION,1, 1, pointer) != 1)
	{
		while(1);	
	}

	//=============================================================================
	//Write Spaces
	//============================================================================= 
	for(spaceCount = 0; spaceCount <= 2; spaceCount++)
	{ 
		if (FSfwrite ((void *)headerSPACE,1, 1, pointer) != 1)
		{
			while(1);	
		}
	}

	//=============================================================================
	//Channel Name
	//=============================================================================  
	if (FSfwrite ((void *)headerNAME,1, 4, pointer) != 4)
	{
		while(1);	
	}

	
	//=============================================================================
	//Write Spaces
	//============================================================================= 
	for(spaceCount = 0; spaceCount <= 11; spaceCount++)
	{ 
		if (FSfwrite ((void *)headerSPACE,1, 1, pointer) != 1)
		{
			while(1);	
		}
	}

	
	//=============================================================================
	//Write Electrode type
	//=============================================================================  
	if (FSfwrite ((void *)headerELECTRODE,1, 16, pointer) != 16)
	{
		while(1);	
	}

	//=============================================================================
	//Write Spaces
	//============================================================================= 
	for(spaceCount = 0; spaceCount <= 63; spaceCount++)
	{ 
		if (FSfwrite ((void *)headerSPACE,1, 1, pointer) != 1)
		{
			while(1);	
		}
	}

	//=============================================================================
	//Write Units
	//=============================================================================  
	if (FSfwrite ((void *)headerUNIT,1, 2, pointer) != 2)
	{
		while(1);	
	}

	//=============================================================================
	//Write Spaces
	//============================================================================= 
	for(spaceCount = 0; spaceCount <= 5; spaceCount++)
	{ 
		if (FSfwrite ((void *)headerSPACE,1, 1, pointer) != 1)
		{
			while(1);	
		}
	}

	//=============================================================================
	//Write Physical Minimum
	//=============================================================================  
	if (FSfwrite ((void *)headerDIGMIN,1, 4, pointer) != 4)
	{
		while(1);	
	}

	//=============================================================================
	//Write Spaces
	//============================================================================= 
	for(spaceCount = 0; spaceCount <= 3; spaceCount++)
	{ 
		if (FSfwrite ((void *)headerSPACE,1, 1, pointer) != 1)
		{
			while(1);	
		}
	}

	//=============================================================================
	//Write Physical Maximum
	//=============================================================================  
	if (FSfwrite ((void *)headerDIGMAX,1, 3, pointer) != 3)
	{
		while(1);	
	}

	//=============================================================================
	//Write Spaces
	//============================================================================= 
	for(spaceCount = 0; spaceCount <= 4; spaceCount++)
	{ 
		if (FSfwrite ((void *)headerSPACE,1, 1, pointer) != 1)
		{
			while(1);	
		}
	}

	//=============================================================================
	//Write Digital Minimum
	//=============================================================================  
	if (FSfwrite ((void *)headerDIGMIN,1, 4, pointer) != 4)
	{
		while(1);	
	}

	//=============================================================================
	//Write Spaces
	//============================================================================= 
	for(spaceCount = 0; spaceCount <= 3; spaceCount++)
	{ 
		if (FSfwrite ((void *)headerSPACE,1, 1, pointer) != 1)
		{
			while(1);	
		}
	}

	//=============================================================================
	//Write Digital Maximum
	//=============================================================================  
	if (FSfwrite ((void *)headerDIGMAX,1, 3, pointer) != 3)
	{
		while(1);	
	}

	//=============================================================================
	//Write Spaces
	//============================================================================= 
	for(spaceCount = 0; spaceCount <= 4; spaceCount++)
	{ 
		if (FSfwrite ((void *)headerSPACE,1, 1, pointer) != 1)
		{
			while(1);	
		}
	}

	//=============================================================================
	//Write Spaces (FILTERING)
	//============================================================================= 
	for(spaceCount = 0; spaceCount <= 79; spaceCount++)
	{ 
		if (FSfwrite ((void *)headerSPACE,1, 1, pointer) != 1)
		{
			while(1);	
		}
	}

	//=============================================================================
	//Write Sample Rate
	//=============================================================================  
	if (FSfwrite ((void *)headerSAMPLERATE,1, 3, pointer) != 3)
	{
		while(1);	
	}

	//=============================================================================
	//Write Spaces
	//============================================================================= 
	for(spaceCount = 0; spaceCount <= 4; spaceCount++)
	{ 
		if (FSfwrite ((void *)headerSPACE,1, 1, pointer) != 1)
		{
			while(1);	
		}
	}

	//=============================================================================
	//Write Reserved
	//=============================================================================  
	if (FSfwrite ((void *)headerRESERVED,1, 8, pointer) != 8)
	{
		while(1);	
	}

	//=============================================================================
	//Write Spaces
	//============================================================================= 
	for(spaceCount = 0; spaceCount <= 23; spaceCount++)
	{ 
		if (FSfwrite ((void *)headerSPACE,1, 1, pointer) != 1)
		{
			while(1);	
		}
	}
	
}

int recombine( unsigned int H, unsigned int L )
{
	return ((H*256)+L);
}





