#include<p33fxxxx.h>
#include<delay.h>
#include<dsp.h>
#include<math.h>
#include "SPIFlash.h"
#include "spi.h"

// ----------------------------------------------
// Data Address Locations in RAM
#define DATA_REAL	0x000000	// 0	 to 16383	16-bit int	data real component
#define DATA_IMAG	0x004000	// 16384 to 32767	16-bit int	data imaginary component
#define TWID_REAL	0x008000	// 32768 to 49151	16-bit int	twiddle factors real
#define TWID_IMAG	0x00C000	// 49152 to 65535	16-bit int	twiddle factors imaginary
#define REVS_ORDR	0x010000	// 65536 to 73727	16-bit int	bit reversed order
#define TEMP_ARRY	0x012000	// 73728 to 81919	16-bit int	store data before reverse sorted
#define OUTP_ARRY	0x014000	// 81920 to 98303	32-bit int	output FFT

// ----------------------------------------------
// for FFT
#define n 4096			// 4096 point FFT
#define t 12			// 12 stages

// ----------------------------------------------
// for Scaling
#define scale_yaxis		0x1BF6
//#define scale_factor	634
// sc4002	681
// sc4012	574
// sc4102	584
// sc4112	634

void FFT(unsigned char sleep_set)
{
	// ****************** FFT VARIABLES ******************

	unsigned long int L;		// number of spectra in each stage
	unsigned long int r;		// length in samples of the spectra in each stage
	unsigned long int L2;		// number of butterfly calculations

	unsigned long int loop;		// loop variable
	unsigned long int q;		// stages number loop
	unsigned long int k;		// spectra number loop
	unsigned long int j;		// butterfly calculations number loop

	unsigned long int kL;		// intermediate index calculation variable
	unsigned long int n1;		// index calculation variable
	unsigned long int n2;		// index calculation variable
	unsigned long int n3;		// index calculation variable
	unsigned long int n3temp;	// ****ADDED

	signed long int tempr;				// butterfly multiplication component temp
	signed long int tempi;				// butterfly multiplication component temp

	unsigned long int ttempr1;
	signed long int ttempr2;
	unsigned long int ttempr3;
	signed long int ttempr4;

	unsigned long int ttempi1;
	signed long int ttempi2;
	unsigned long int ttempi3;
	signed long int ttempi4;
	
	
	// ----------------------------------------------
	signed int xrN2 = 0;
	signed int xiN2 = 0;
	signed int xrN3 = 0;
	signed int xiN3 = 0;
	signed int wrN1 = 0;
	signed int wiN1 = 0;

	unsigned char xrN2s[2];
	unsigned char xiN2s[2];
	unsigned char xrN3s[2];
	unsigned char xiN3s[2];
	unsigned char wrN1s[2];
	unsigned char wiN1s[2];	

	unsigned char xrneg_flag = 0;
	unsigned char xineg_flag = 0;
	unsigned char wrneg_flag = 0;
	unsigned char wineg_flag = 0;

	unsigned char typeMAX_flag = 0;

	unsigned int count = 0;			// **** DEBUG
	unsigned char variable;			// **** DEBUG

	// **************** SCALE VARIABLES ****************
	unsigned long int output_temp;
	signed int XR;
	signed int XI;

	unsigned char XRs[2];
	unsigned char XIs[2];

	unsigned char OUTs[2];

	int scale_factor = 0;

	// ***************************************************************	
	// determine scale_factor
	switch(sleep_set)
	{
		// sc4002	681
		// sc4012	574
		// sc4102	584
		// sc4112	634
		case 1:
		{
			scale_factor = 681;
			break;
		}
		case 2:
		{
			scale_factor = 574;			
			break;
		}
		case 3:
		{
			scale_factor = 584;
			break;
		}
		case 4:
		{
			scale_factor = 634;
			break;
		}
	}

	// **************************************************************************************************************	
	// begin FFT calculation

	SPIFlashBeginWrite(DATA_REAL);

	// ******* STAGES LOOP ******** ( 12 stages )
	for(q = 1; q <= t; q++)		
	{
		L = 1; L <<= q;			// 2^q stages (length each spectra)
		r = 1; r <<= (t - q);	// samples in each stage (samples each spectra)
		L2 = L >> 1;			// butterfly calculations

		kL = 0;

		// ******* SPECTRA LOOP ******** ( 0,1,2,3,4,5... < 2048, 1024, 512... )
		for(k = 0; k < r; k++)
		{
			// ******* BUTTERFILES ******** ( 0,1,2,3,4,5... < 1024, 512, 256... )
			for(j = 0; j < L2; j++)
			{
				n3temp = kL + j;		// determine array indexes
				n3 = n3temp * 2;
				n2 = (n3temp + L2) * 2;
				n1 = (L2 - 1 + j) * 2;		// ****changed -1 to -2
				/*
				// butterfly:
				// A = a + twiddles*b
				// B = b - twiddles*b
				// so first twiddles*b (both complex)
				// (inputR + inputI)(twiddlesR + twiddlesI)
				*/
				SPIFlashReadArray((TWID_REAL + n1), wrN1s, 2);
				SPIFlashReadArray((TWID_IMAG + n1), wiN1s, 2);
				SPIFlashReadArray((DATA_REAL + n2), xrN2s, 2);
				SPIFlashReadArray((DATA_IMAG + n2), xiN2s, 2);
				SPIFlashReadArray((DATA_REAL + n3), xrN3s, 2);
				SPIFlashReadArray((DATA_IMAG + n3), xiN3s, 2);
				
				wrN1 = ((int)wrN1s[0] << 8); wrN1 ^= (int)wrN1s[1];				
				wiN1 = ((int)wiN1s[0] << 8); wiN1 ^= (int)wiN1s[1];				
				xrN2 = ((int)xrN2s[0] << 8); xrN2 ^= (int)xrN2s[1];				
				xiN2 = ((int)xiN2s[0] << 8); xiN2 ^= (int)xiN2s[1];
				xrN3 = ((int)xrN3s[0] << 8); xrN3 ^= (int)xrN3s[1];				
				xiN3 = ((int)xiN3s[0] << 8); xiN3 ^= (int)xiN3s[1];

				// ------------ FIX ARITHMETIC OVERFLOW ISSUE ------------

				// **** DEBUG
				//if(count == 0x801)
				//{
				//	variable = 0;
				//}

				// test for negative
				if((xrN2 >> 15) & 0x01){ xrN2 *= -1; xrneg_flag = 1;}
				if((xiN2 >> 15) & 0x01){ xiN2 *= -1; xineg_flag = 1;}	
				if((wrN1 >> 15) & 0x01){ wrN1 *= -1; wrneg_flag = 1;}
				if((wiN1 >> 15) & 0x01){ wiN1 *= -1; wineg_flag = 1;}

				// test for 0x8000
				if(xrN2 == 0x8000){ typeMAX_flag = 1;}
				if(xiN2 == 0x8000){ typeMAX_flag = 1;}	
				if(wrN1 == 0x8000){ typeMAX_flag = 1;}
				if(wiN1 == 0x8000){ typeMAX_flag = 1;}


				// TEMP REAL *****************************************************
				ttempr1 = (unsigned long int)wrN1 * (unsigned long int)xrN2;
				if((wrneg_flag || xrneg_flag) && (wrneg_flag != xrneg_flag))
				{ 
					ttempr1 *= -1;
				}

				ttempr2 = (signed long int)ttempr1;
				ttempr2 >>= 15;			
				
				ttempr3 = (unsigned long int)wiN1 * (unsigned long int)xiN2;
				if((wineg_flag || xineg_flag) && (wineg_flag != xineg_flag))
				{
					ttempr3 *= -1;
				}

				ttempr4 = (signed long int)ttempr3;
				ttempr4 >>= 15;

				tempr = ttempr2 - ttempr4;

				// **** DEBUG
				//if(count == 0x801)
				//{
				//	variable = 0;
				//}

				// TEMP IMAG *****************************************************
				ttempi1 = (unsigned long int)wrN1 * (unsigned long int)xiN2;
				if((wrneg_flag || xineg_flag) && (wrneg_flag != xineg_flag))
				{ 
					ttempi1 *= -1;
				}

				ttempi2 = (signed long int)ttempi1;
				ttempi2 >>= 15;

				ttempi3 = (unsigned long int)wiN1 * (unsigned long int)xrN2;
				if((wineg_flag || xrneg_flag) && (wineg_flag != xrneg_flag))
				{
					ttempi3 *= -1;
				}

				ttempi4 = (signed long int)ttempi3;
				ttempi4 >>= 15;

				tempi = ttempi2 + ttempi4;

				// ***************************************************************				


				xrN2 = ((long int)xrN3 - tempr) >> 1;
				//if(typeMAX_flag){ xrN2 *= -1; }
				xiN2 = ((long int)xiN3 - tempi) >> 1;
				if(typeMAX_flag)
				{
					xiN2 *= -1;
					xiN2 -= 1;
				}
				xrN3 = ((long int)xrN3 + tempr) >> 1;
				//if(typeMAX_flag){ xrN3 *= -1; }
				xiN3 = ((long int)xiN3 + tempi) >> 1;
				if(typeMAX_flag)
				{ 
					xiN3 *= -1;
					xiN3 -= 1;
				}	
				
				// **** DEBUG
				//if(count == 0x801)
				//{
				//	variable = 0;
				//}	

				xrN2s[0] = (xrN2 >> 8);	xrN2s[1] = xrN2;
				xiN2s[0] = (xiN2 >> 8);	xiN2s[1] = xiN2;
				xrN3s[0] = (xrN3 >> 8);	xrN3s[1] = xrN3;
				xiN3s[0] = (xiN3 >> 8);	xiN3s[1] = xiN3;							

				SPIFlashBeginWrite(DATA_REAL + n2);	SPIFlashWrite(xrN2s[0]); SPIFlashWrite(xrN2s[1]);
				SPIFlashBeginWrite(DATA_IMAG + n2);	SPIFlashWrite(xiN2s[0]); SPIFlashWrite(xiN2s[1]);
				SPIFlashBeginWrite(DATA_REAL + n3);	SPIFlashWrite(xrN3s[0]); SPIFlashWrite(xrN3s[1]);
				SPIFlashBeginWrite(DATA_IMAG + n3);	SPIFlashWrite(xiN3s[0]); SPIFlashWrite(xiN3s[1]);

				xrneg_flag = 0;
				xineg_flag = 0;	
				wrneg_flag = 0;
				wineg_flag = 0;	

				typeMAX_flag = 0;


				// **** DEBUG
				//if(count == 0xFFF)
				//{
				//	variable = 0;
				//}
				// **** DEBUG
				//if(xiN2 == 0xffc1 || xiN3 == 0xffc1)
				//{
				//	variable = 0;
				//}	
	
				// ****DEBUG
				count += 1;

			}

			kL += L;
		}
	}

	// ***************************************************
	// find absolute value and do scaling
	for(loop = 0; loop < n; loop++)
	{
		//output_temp = pow(xr[loop],2) + pow(xi[loop],2);
		//output[loop] = (output_temp * scale_yaxis) >> 16;		// y-axis in uV/Hz
		//output[loop] = (output[loop] * scale_factor) >> 10;		// subtract base divide by gain scaling
		//output[loop] *=2;										// shift by 14 instead of 15 does -> single sided so multiply by 2
																// shift by 10 do the addition and then right shift values off by 4
		SPIFlashReadArray((DATA_REAL + (loop * 2)), XRs, 2);
		XR = ((int)XRs[0] << 8);
		XR ^= (int)XRs[1];

		SPIFlashReadArray((DATA_IMAG + (loop * 2)), XIs, 2);
		XI = ((int)XIs[0] << 8);
		XI ^= (int)XIs[1];

		output_temp = pow((long int)XR, 2) + pow((long int)XI, 2);
		output_temp = (output_temp * scale_yaxis) >> 16;
		output_temp = (output_temp * scale_factor) >> 10;
		
		OUTs[0] = (output_temp >> 8);	OUTs[1] = output_temp;

		SPIFlashBeginWrite(OUTP_ARRY + (loop * 2));	SPIFlashWrite(OUTs[0]); SPIFlashWrite(OUTs[1]);
	}
	//variable = 0;
}
