#include<p33fxxxx.h>
#include<uart.h>
#include<delay.h>
#include<dsp.h>
#include "SPIFlash.h"
#include "spi.h"
#include "parse_packet.h"
#include "configure_UART.h"
#include "FFT.h"
#include "neural_network.h"

// ----------------------------------------------
// Data Address Locations in RAM
#define DATA_REAL	0x000000	// 0	 to 16383	16-bit int	data real component
#define DATA_IMAG	0x004000	// 16384 to 32767	16-bit int	data imaginary component
#define TWID_REAL	0x008000	// 32768 to 49151	16-bit int	twiddle factors real
#define TWID_IMAG	0x00C000	// 49152 to 65535	16-bit int	twiddle factors imaginary
#define REVS_ORDR	0x010000	// 65536 to 73727	16-bit int	bit reversed order
#define TEMP_ARRY	0x012000	// 73728 to 81919	16-bit int	store data before reverse sorted
#define OUTP_ARRY	0x014000	// 81920 to 98303	32-bit int	output FFT

// ----------------------------------------------
// for FFT
#define n 4096			// 4096 point FFT
#define t 12			// 12 stages

// ----------------------------------------------
// Configuration
_FOSCSEL(FNOSC_FRC);
_FOSC(	FCKSM_CSECMD &
		OSCIOFNC_OFF & 
		POSCMD_XT);
_FWDT(	FWDTEN_OFF);							// watchdog timer off
_FGS(GCP_OFF);									// turn off code protect
_FPOR(FPWRT_PWR1);								// turn off power up timer

// ----------------------------------------------
// Global Variables (Used in Parse Packet)
unsigned char state = 0;
unsigned char count = 0;
unsigned char data = 0;
int flag = 0;				

unsigned char data_set = 0;
int byte_count = 0;			// for the capstone demo

// *********************************************************************************
// M A I N
// *********************************************************************************
int main(void)
{
	// Configure PLL prescaler, PLL postscaler, PLL divisor
	PLLFBD=30; 									// M = 32
	CLKDIVbits.PLLPOST = 0;						// N2 = 2
	CLKDIVbits.PLLPRE = 0; 						// N1 = 2

		// Initiate Clock Switch to Primary Oscillator with PLL (NOSC = 0b011)
		__builtin_write_OSCCONH(0x03);
		__builtin_write_OSCCONL(0x01);

		// Wait for Clock switch to occur
		while (OSCCONbits.COSC != 0b011);

		// Wait for PLL to lock
		while(OSCCONbits.LOCK != 1) {};

	// ----------------------------------------------
	// configure I/O
	ADPCFG = 0xFFFF;							// all pins digital	
	TRISA = 0b00000000;							// PORTA output
	//TRISB = 0b1110011111100111;					// RB2 input for UART, RB3, RB11, and RB12 output
	//TRISB = 0b11110111;							// RB2 input for UART, RB3 output

	TRISBbits.TRISB2 = 1;
	TRISBbits.TRISB3 = 0;	
	//TRISBbits.TRISB6 = 0;
	//TRISBbits.TRISB7 = 0;
	TRISBbits.TRISB4 = 0;
	TRISBbits.TRISB11 = 0;
	TRISBbits.TRISB12 = 0;

	//CMCONbits.C1OUTEN = 0;
	//CMCONbits.C2OUTEN = 0;
	

	// ----------------------------------------------
	// configure UART
	configureUART();

	// ----------------------------------------------
	// configure SPI
	RPINR20bits.SDI1R = 0b01101;				// set RB13 to input for SPI RX
	RPOR6bits.RP12R = 0b00111;					// set RB12 to output for SPI TX
	RPOR5bits.RP11R = 0b01000;					// SPI1 Clock Output

	SPIFlashInit();

	// ********** BIT REVERSE ARRAY VARIABLES ************

	unsigned char brdat[2];
	unsigned char brrev[2];
	unsigned long int braddr;

	// ****************** FOR FEATURES *******************

	unsigned long int F1 = 0;			// index 17 to 63
	unsigned long int F2 = 0;			// index 64 to 131
	unsigned long int F3 = 0;			// index 132 to 352
	unsigned long int F4 = 0;			// index 353 to 450
	unsigned long int F5 = 0;			// index 451 to 638
	unsigned long int F6 = 0;			// index 639 to 901
	unsigned long int F7 = 0;			// index 902 to 1535
	unsigned long int F8 = 0;			// total energy

	double F9 = 0;					// delta
	double F10 = 0;					// alpha
	double F11 = 0;					// kk & spindles
	double F12 = 0;					// alpha / theta
	double F13 = 0;					// delta / theta

	unsigned char tempF1[2];
	unsigned int tempF2 = 0;

	// ************** FOR LOG TRANSFORM ******************

	double log_features[13];

	// ************** FOR NEURAL NETWORK *****************
	
	unsigned char state = 0;
	double sleep_state[6];						// sleep state
	double *plog_features = log_features;
	double *psleep_state = sleep_state;

	// ----------------------------------------------
	// transfer data to RAM registers 	--<< LOAD NEW FILE INTO MEMORY >>--
	SPIFlashBeginWrite(TEMP_ARRY);				// start at this address

	// *********** UART TEST RESULTS VARIABLES ***********
	unsigned char b;
	unsigned int loop;
	unsigned long int address = OUTP_ARRY;
	// ***************************************************
/*
	// ***************************************************
	// UART TX watch window 			--<< FOR DEBUGGING >>--
	for(loop = 0; loop < 8192; loop++)
	{
		SPIFlashReadArray(address, &b, 1);
		address += 1;
		U2TXREG = b;
		while(U2STAbits.UTXBF){};
	}
*/
	// **************************************************************************************************************
	// ----------------------------------------- C O D E  S T A R T S  H E R E---------------------------------------
	// **************************************************************************************************************

	LATBbits.LATB4 = 1;
	LATAbits.LATA1 = 1;
	LATAbits.LATA0 = 1;

	while(1)
	{

		// ***************************************************
		// When transmission is complete
		if(byte_count == 0x2000)
		{
			LATBbits.LATB4 = 1;				// turn LEDs off
			LATAbits.LATA1 = 1;				// turn LEDs off
			LATAbits.LATA0 = 1;				// turn LEDs off
			
			IEC0bits.U1RXIE = 0;			// disable RX interupt
			byte_count = 0;					// reset byte_count

			// ***************************************************
			// populate imaginary component of input with zeros
			SPIFlashBeginWrite(DATA_IMAG);

			for(loop = 0; loop < 8192; loop++)
			{
				SPIFlashWrite(0x0000);
			}

			// ***************************************************
			// bit reverse order array
			SPIFlashBeginWrite(DATA_REAL);

			for(loop = 0; loop < n; loop++)
			{
				SPIFlashReadArray((REVS_ORDR + (loop * 2)), brrev, 2);
				braddr = ((long int)brrev[0] << 8);
				braddr ^= (long int)brrev[1];
				SPIFlashReadArray((TEMP_ARRY + (braddr * 2)), brdat, 2);
				SPIFlashWrite(brdat[0]);
				SPIFlashWrite(brdat[1]);		
			}	
	
			// ***************************************************
			// this is where the magic happens
			FFT(data_set);
			// ***************************************************

			// ***************************************************
			// calculate features
			for(loop = 17; loop < 64; loop++)
			{
				SPIFlashReadArray((OUTP_ARRY + (loop * 2)), tempF1, 2);
				tempF2 = ((int)tempF1[0] << 8);
				tempF2 ^= (int)tempF1[1];		
				F1 += tempF2;
			}
			F1 = F1 >> 4;
	
			for(loop = 64; loop < 132; loop++)
			{
				SPIFlashReadArray((OUTP_ARRY + (loop * 2)), tempF1, 2);
				tempF2 = ((int)tempF1[0] << 8);
				tempF2 ^= (int)tempF1[1];
				F2 += tempF2;
			}
			F2 = F2 >> 4;
		
			for(loop = 132; loop < 353; loop++)
			{
				SPIFlashReadArray((OUTP_ARRY + (loop * 2)), tempF1, 2);
				tempF2 = ((int)tempF1[0] << 8);
				tempF2 ^= (int)tempF1[1];
				F3 += tempF2;
			}
			F3 = F3 >> 4;
		
			for(loop = 353; loop < 451; loop++)
			{
				SPIFlashReadArray((OUTP_ARRY + (loop * 2)), tempF1, 2);
				tempF2 = ((int)tempF1[0] << 8);
				tempF2 ^= (int)tempF1[1];
				F4 += tempF2;
			}
			F4 = F4 >> 4;
		
			for(loop = 451; loop < 639; loop++)
			{
				SPIFlashReadArray((OUTP_ARRY + (loop * 2)), tempF1, 2);
				tempF2 = ((int)tempF1[0] << 8);
				tempF2 ^= (int)tempF1[1];
				F5 += tempF2;
			}
			F5 = F5 >> 4;
		
			for(loop = 639; loop < 902; loop++)
			{
				SPIFlashReadArray((OUTP_ARRY + (loop * 2)), tempF1, 2);
				tempF2 = ((int)tempF1[0] << 8);
				tempF2 ^= (int)tempF1[1];
				F6 += tempF2;
			}
			F6 = F6 >> 4;
		
			for(loop = 902; loop < 1536; loop++)
			{
				SPIFlashReadArray((OUTP_ARRY + (loop * 2)), tempF1, 2);
				tempF2 = ((int)tempF1[0] << 8);
				tempF2 ^= (int)tempF1[1];
				F7 += tempF2;
			}
			F7 = F7 >> 4;
		
			F8 = F1 + F2 + F3 + F4 + F5 + F6 + F7;
			F9 = ((double)F1 + (double)F2) / (double)F8;
			F10 = (double)F4 / (double)F8;
			F11 = ((double)F1 + (double)F5) / (double)F8;
			F12 = (double)F4 / (double)F3;
			F13 = ((double)F1 + (double)F2) / (double)F3;
		
			// ***************************************************
			// log transform features
		
			log_features[0] = log((double)F1);
			log_features[1] = log((double)F2);
			log_features[2] = log((double)F3);
			log_features[3] = log((double)F4);
			log_features[4] = log((double)F5);
			log_features[5] = log((double)F6);
			log_features[6] = log((double)F7);
			log_features[7] = log((double)F8);
			log_features[8] = log(1-(double)F9);
			log_features[9] = log(F10);
			log_features[10] = F11;
			log_features[11] = log(F12);
			log_features[12] = log(F13);
			
			// ***************************************************
			// run neural network
			state = determine_state(plog_features, psleep_state);
			//loop = 0;		// **** DEBUG

			// ***************************************************
			// Display sleep state on LEDs
			switch(state)
			{
				case 0:
				{
					LATBbits.LATB4 = 0;
					LATAbits.LATA1 = 1;
					LATAbits.LATA0 = 1;
					break;
				}
				case 1:
				{
					LATBbits.LATB4 = 1;
					LATAbits.LATA1 = 0;
					LATAbits.LATA0 = 1;
					break;
				}
				case 2:
				{
					LATBbits.LATB4 = 0;
					LATAbits.LATA1 = 0;
					LATAbits.LATA0 = 1;
					break;
				}
				case 3:
				{
					LATBbits.LATB4 = 1;
					LATAbits.LATA1 = 1;
					LATAbits.LATA0 = 0;
					break;
				}
				case 4:
				{
					LATBbits.LATB4 = 0;
					LATAbits.LATA1 = 1;
					LATAbits.LATA0 = 0;
					break;
				}
				case 5:
				{
					LATBbits.LATB4 = 1;
					LATAbits.LATA1 = 0;
					LATAbits.LATA0 = 0;
					break;
				}
				default:
				{
					LATBbits.LATB4 = 1;
					LATAbits.LATA1 = 1;
					LATAbits.LATA0 = 1;
					break;
				}
				
			}

			// ***************************************************

			data_set = 0;
			SPIFlashBeginWrite(TEMP_ARRY);				// start at this address
			IEC0bits.U1RXIE = 1;						// enable RX interupt
		}

	}

	return 0;
}

// *********************************************************************************
// U A R T  R E C E I V E  I N T E R R U P T
// *********************************************************************************
void __attribute__((interrupt, no_auto_psv)) _U1RXInterrupt(void)
{
	
	parse_packet();
	IFS0bits.U1RXIF = 0;						// clear RX interrupt status flag
}

